package fixstyle;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;

import parser.ASTCompilationUnit;
import parser.JavaParser;
import parser.JavaParserVisitor;
import parser.ParseException;

/*
 * ToDo
 * ====
 * Fixes:
 * Add/remove final method parameters...
 * Remove unused modifiers from interfaces...
 * Move square brackets to the left in array type declarations...
 * Add braces onto if/do/while/for statements that lack them
 * Expand code substantially to handle formatting and indentation...
 * 
 * Done
 * ====
 * Sort modifiers into the correct order...
 */
public class FixStyle
{
    public static void main(String args[])
    {
        if (((args.length) > 2) || ((args.length) < 1)) {
            usage();
            return;
        } 
        
        try
        {
            String in_path = args[0];
            String out_path = args[1];
            process(in_path, out_path);
        } catch (Exception e)
        {
            System.err.println("A problem has arisen:");
            System.err.println("" + e);
            e.printStackTrace();
        }
    } 
    
    static void usage() {
        System.err.println("FixStyle: a Java source code modifier that attempts");
        System.err.println("to automatically correct stylistic problems.");
        System.err.println("Usage: FixStyle <infile> <outfile>");
        System.err.println("If a directory is supplied, all the java files");
        System.err.println("inside it will be processed recursively.");
    }

    public static void process(String in_path, String out_path)
        throws FileNotFoundException, IOException, ParseException
    {
        File f_in = new File(in_path);
        File f_out = new File(out_path);

        String abs_in_path = f_in.getAbsolutePath();
        String abs_out_path = f_out.getAbsolutePath();

        if (f_in.isDirectory())
        {
            // process recursively...

            processDirectory(abs_in_path, abs_out_path);
        } else
        {
            // process individual...

            processFile(abs_in_path, abs_out_path);
        }
    }

    private static void processDirectory(
        String abs_in_path,
        String abs_out_path)
        throws FileNotFoundException, IOException, ParseException
    {
        int abs_in_path_length = abs_in_path.length();
        DirectoryRecursor dr = new DirectoryRecursor();

        File[] file_array = dr.scanDirectory(abs_in_path);
        for (int i = 0; i < file_array.length; i++)
        {
            String in_file_path = file_array[i].getAbsolutePath();
            String upper_branch = in_file_path.substring(abs_in_path_length);
            String out_file_path = abs_out_path + upper_branch;
            makeDirectoriesIfNecessary(out_file_path);
            processFile(in_file_path, out_file_path);
        }
    }

    private static void makeDirectoriesIfNecessary(String out_file_path)
    {
        File f = new File(out_file_path);
        String parent = f.getParent();
        File parent_file = new File(parent);
        parent_file.mkdirs();
    }

    static void processFile(String in_path, String out_path)
        throws FileNotFoundException, IOException, ParseException
    {
        FileInputStream in = null;
        OutputStream out = null;
        ASTCompilationUnit cu = null;
        
        try
        {
            in = new FileInputStream(in_path);
            JavaParser p = new JavaParser(in);

            cu = p.CompilationUnit();
        } finally
        {
            in.close();
        }

        try
        {
            ArrayList list = new ArrayList();

            ModifiableTokenStreamMaker mts =
                new ModifiableTokenStreamMaker(list);
            cu.jjtAccept((JavaParserVisitor) mts, null);

            ModifiableTokenStreamProcessor processor =
                new ModifiableTokenStreamProcessor(list);
            processor.process();

            out = new FileOutputStream(out_path);
            ModifiableTokenStreamPrinter.print(list, out);
        } finally
        {
            out.flush();
            out.close();
        }
    }
}
